;;;; hashes.scm
;;;; Kon Lovett, Jan '06

;; Issues
;;
;; - CRC hash accumulation technique is questionable.

(use message-digest box crc hash-utils mathh-int hashes-support)

(declare
  (usual-integrations)
  (inline)
  (disable-interrupts)
  (no-procedure-checks)
  (no-bound-checks)
  (export
    CRCHash-prim
    CRCHash
    CRCHash:binary-digest CRCHash:digest CRCHash:primitive ) )

;;;

(define (CRCHash-prim str len hash)
	(let ([crc (crc32-byte str len)])
		(if hash
			(bitwise-xor crc hash)
			crc ) ) )

(define (CRCHash str . args)
  (let-optionals args ([len (string-length str)] [seed #f])
    (CRCHash-prim str len seed) ) )

;;;

(define (CRCHash-context)
	(make-box) )

(define (CRCHash-init ctx)
	(box-set! ctx #f) )

(define (CRCHash-update ctx str len)
	(box-set! ctx (CRCHash-prim str len (box-ref ctx))) )

(define (CRCHash-final ctx result)
	(unsigned-integer32-set! result (box-ref ctx)) )

;;;

(define (CRCHash:binary-digest obj)
	(make-binary-message-digest obj
		CRCHash-context unsigned-integer32-size
		CRCHash-init CRCHash-update CRCHash-final) )

(define (CRCHash:digest obj)
	(make-message-digest obj
		CRCHash-context unsigned-integer32-size
		CRCHash-init CRCHash-update CRCHash-final) )

(define (CRCHash:primitive)
	(make-message-digest-primitive
		CRCHash-context	unsigned-integer32-size
		CRCHash-init CRCHash-update CRCHash-final))
