;;;; TWUserMixHash.scm
;;;; Kon Lovett, Feb '08

(use srfi-12)
(use TWUserMixHash-support hash-utils)

(declare
  (usual-integrations)
  (inline)
  (no-procedure-checks)
  (no-bound-checks)
  (export
    make-TWUserMixHash-primitive-procedure
    make-TWUserMixHash ) )

;;;

(define-inline (check-procedure loc obj)
  (##sys#check-closure obj loc) )

;;;

;; Signatures:
;;
;; mix-procedure
;; (unsigned-integer32 -> unsigned-integer32)
;;
;; hash-primitive-procedure
;; (scheme-object unsigned-integer32 unsigned-integer32 -> unsigned-integer32)
;;
;; hash-procedure
;; (scheme-object #!optional unsigned-integer32 unsigned-integer32 -> unsigned-integer32)

;; Takes 1 argument:
;; mix-procedure
;;
;; Returns 1 value:
;; hash-primitive-procedure

(define (make-TWUserMixHash-primitive-procedure mix-proc #!optional unsafe?)
  (check-procedure 'make-TWUserMixHash-primitive-procedure mix-proc)
  (if unsafe?
      (cut foreign-TWUserMixHash <> <> <> mix-proc)
      (let* ([*last-exception* #f]
             [mixer (lambda (key)
                      (handle-exceptions ex
                          (begin (set! *last-exception* ex) 0)
                        (set! *last-exception* #f)
                        (mix-proc key) ) ) ] )
        (lambda (data length initval)
          (let ([key (foreign-TWUserMixHash data length initval mixer)])
            (if *last-exception*
                (abort *last-exception*)
                key ) ) ) ) ) )

;; Takes 1 argument:
;; mix-procedure
;;
;; Returns 5 values:
;; hash-primitive-procedure
;; hash-procedure
;; binary-message-digest
;; message-digest
;; message-digest-primitive

(define (make-TWUserMixHash mix-proc #!optional unsafe?)
  (check-procedure 'make-TWUserMixHash mix-proc)
  (let ([prim-proc (make-TWUserMixHash-primitive-procedure mix-proc unsafe?)])
      (apply values prim-proc
                    (make-hash-procedure prim-proc)
                    (make-hash-message-digest-procedures prim-proc)) ) )
