;;;; hashes-macros.scm
;;;; Kon Lovett, Aug '06

;;;; An Include File

(use mathh-int)

(define-for-syntax (gen-proc-sym name suff)
  (string->symbol (string-append (symbol->string name) suff)) )

(define-for-syntax (gen-prim-proc-sym name)
  (gen-proc-sym name "-prim") )

(define-for-syntax (gen-update-proc-sym name)
  (gen-proc-sym name "-update") )

(define-macro (gen-hash-api NAME)
  `(begin
     (define ,(gen-prim-proc-sym NAME)
       (foreign-lambda unsigned-integer32 ,(symbol->string NAME) scheme-pointer unsigned-integer32 unsigned-integer32) )
     (define ,NAME
       (let ([byte-length string-length])
         (lambda (dat . args)
           (let-optionals args ([len (byte-length dat)] [initval 0])
             (,(gen-prim-proc-sym NAME) dat len initval)) ) ) ) ) )

(define-macro (gen-update-proc NAME)
  `(define ,(gen-update-proc-sym NAME)
     (foreign-lambda* void ((c-pointer ctx) (scheme-pointer dat) (unsigned-integer32 len))
       ,(conc "((hashctx *) ctx)->hash = " NAME " ((uint8_t *) dat, (uint32_t) len, ((hashctx *) ctx)->hash);")) ) )

(define-macro (gen-md-api NAME)
  (let ([BDN  (gen-proc-sym NAME ":binary-digest")]
        [DN   (gen-proc-sym NAME ":digest")]
        [PN   (gen-proc-sym NAME ":primitive")]
        [UN   (gen-update-proc-sym NAME)])
    `(begin
       (define (,BDN obj)
         (make-binary-message-digest obj
           hashes:hash-context-size unsigned-integer32-size
           hashes:generic-init ,UN hashes:generic-final
           ',BDN) )
       (define (,DN obj)
         (make-message-digest obj
           hashes:hash-context-size unsigned-integer32-size
           hashes:generic-init ,UN hashes:generic-final
           ',DN) )
       (define (,PN)
         (make-message-digest-primitive
           hashes:hash-context-size unsigned-integer32-size
           hashes:generic-init ,UN hashes:generic-final
           ',PN)) ) ) )

(define-constant DEFAULT-HASH-SEED 0)
