;;;; hashes-support.scm
;;;; Kon Lovett, Jan '06

;; Issues
;;
;; - This is cavalier w/ alignment, assumes uint8_t * arguments on
;; a 32-bit boundry. Chicken heap & nursery object data are so
;; constrained.

;; TBD
;;
;; - LITTLE/BIG-ENDIAN versions of all hash functions are not provided.
;;
;; - 64-bit versions of alll hash functions are not provided.
;;
;; - 64-bit optimization: Chicken object data is 64-bit aligned so
;; could process data to hash in 64-bit chunks, rather than 32-bit.
;;
;; - Better initval handling, what ever that means.

(declare
  (uses lolevel)
  (usual-integrations)
  (inline)
  (generic)
  (disable-interrupts)
  (no-procedure-checks)
  (no-argc-checks)
  (no-bound-checks)
  (export
    hs$hash-seed
    unsigned-integer32-set!
    unsigned-integer32-ref
    hashes:hash-context-size
    hashes:ctx-hash-ref
    hashes:ctx-hash-set!
    hashes:generic-init
    hashes:generic-final ) )

#>
#include "hashes.h"

#undef bitsizeof
<#

(include "hashes-macros")

;;;

(define uint32-cptr-ref
  (foreign-lambda* unsigned-integer32 ((c-pointer dat))
    "return (*((uint32_t *) dat));"))

(define uint32-cptr-set!
  (foreign-lambda* void ((c-pointer dat) (unsigned-integer32 w32))
    "*((uint32_t *) dat) = (uint32_t) w32;"))

(define uint32-sptr-ref
  (foreign-lambda* unsigned-integer32 ((scheme-pointer dat))
    "return (*((uint32_t *) dat));"))

(define uint32-sptr-set!
  (foreign-lambda* void ((scheme-pointer dat) (unsigned-integer32 w32))
    "*((uint32_t *) dat) = (uint32_t) w32;"))

(define-inline (ptr-prc-for-obj obj cptr-proc sptr-proc)
    (if (or (pointer? obj) (locative? obj))
        cptr-proc
        sptr-proc ) )

;;;

(define hs$hash-seed DEFAULT-HASH-SEED)

;;;

(define (unsigned-integer32-set! obj num)
  ((ptr-prc-for-obj obj uint32-cptr-set! uint32-sptr-set!) obj num) )

(define (unsigned-integer32-ref obj)
  ((ptr-prc-for-obj obj uint32-cptr-ref uint32-sptr-ref) obj) )

(define hashes:hash-context-size (foreign-value "sizeof (hashctx)" int))

;;

(define hashes:ctx-hash-ref
  (foreign-lambda* unsigned-integer32 ((c-pointer ctx))
   "return (((hashctx *) ctx)->hash);") )

;;

(define hashes:ctx-hash-set!
  (foreign-lambda* void ((c-pointer ctx) (unsigned-integer32 val))
   "((hashctx *) ctx)->hash = val;"))

;;

(define (hashes:generic-init ctx)
	(hashes:ctx-hash-set! ctx hs$hash-seed) )

;;

(define (hashes:generic-final ctx result)
	(unsigned-integer32-set! result (hashes:ctx-hash-ref ctx)) )
