;;;; hashes-test.scm

(use testbase testbase-output-human)
(use hashes)
(use rabin-karp)
(use srfi-1)

;;;

(define-expect-unary procedure?)
(define-expect-unary fixnum?)
(define-expect-unary number?)
(define-expect-binary >)

;;;

(define TSTSTR "The large brown fox jumped over the lazy dog.")

(define TSTSTR-LEN        (string-length TSTSTR))
(define TSTSTR-HALF-LEN   (fx/ TSTSTR-LEN 2))

;;;

(define-test hashes-test "Hash Functions"

	(test/suite "Hash-prim Half Length"

		(expect-eqv "RJMXHash"      1495738488  (RJMXHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "TWMXHash"      1783737257  (TWMXHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "TWSHMXHash"    4294724170  (TWSHMXHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "TWSHMLMXHash"  3316553819  (TWSHMLMXHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "TWMGMXHash"    3799230039  (TWMGMXHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "RSHash"        561364390   (RSHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "JSHash"        1096595314  (JSHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "PJWHash"       11905       (PJWHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "ELFHash"       125889045   (ELFHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "BKDRHash"      2233812262  (BKDRHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "SDBMHash"      1850315706  (SDBMHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "DJBHash"       664891301   (DJBHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "NDJBHash"      786919305   (NDJBHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "DEKHash"       4159618807  (DEKHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "APHash"        3373256484  (APHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "CRCHash"       4140400781  (CRCHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "PHSFHash"      4232571984  (PHSFHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "FNVHash"       2129953783  (FNVHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "FNVAHash"      1173052123  (FNVAHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "BRPHash"       1793202933  (BRPHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "PYHash"        10752       (PYHash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "RJL3Hash"      1304064403   (RJL3Hash-prim TSTSTR TSTSTR-HALF-LEN 0))
		(expect-eqv "ISPLHash"      2015390325  (ISPLHash-prim TSTSTR TSTSTR-HALF-LEN 0))
	)

	(test/suite "Hash Full Length"

		(expect-eqv "RJMXHash" (RJMXHash-prim TSTSTR TSTSTR-LEN 0) (RJMXHash TSTSTR))
		(expect-eqv "TWMXHash" (TWMXHash-prim TSTSTR TSTSTR-LEN 0) (TWMXHash TSTSTR))
		(expect-eqv "TWSHMXHash" (TWSHMXHash-prim TSTSTR TSTSTR-LEN 0) (TWSHMXHash TSTSTR))
		(expect-eqv "TWSHMLMXHash" (TWSHMLMXHash-prim TSTSTR TSTSTR-LEN 0) (TWSHMLMXHash TSTSTR))
		(expect-eqv "TWMGMXHash" (TWMGMXHash-prim TSTSTR TSTSTR-LEN 0) (TWMGMXHash TSTSTR))
		(expect-eqv "RSHash" (RSHash-prim TSTSTR TSTSTR-LEN 0) (RSHash TSTSTR))
		(expect-eqv "JSHash" (JSHash-prim TSTSTR TSTSTR-LEN 0) (JSHash TSTSTR))
		(expect-eqv "PJWHash" (PJWHash-prim TSTSTR TSTSTR-LEN 0) (PJWHash TSTSTR))
		(expect-eqv "ELFHash" (ELFHash-prim TSTSTR TSTSTR-LEN 0) (ELFHash TSTSTR))
		(expect-eqv "BKDRHash" (BKDRHash-prim TSTSTR TSTSTR-LEN 0) (BKDRHash TSTSTR))
		(expect-eqv "SDBMHash" (SDBMHash-prim TSTSTR TSTSTR-LEN 0) (SDBMHash TSTSTR))
		(expect-eqv "DJBHash" (DJBHash-prim TSTSTR TSTSTR-LEN 0) (DJBHash TSTSTR))
		(expect-eqv "NDJBHash" (NDJBHash-prim TSTSTR TSTSTR-LEN 0) (NDJBHash TSTSTR))
		(expect-eqv "DEKHash" (DEKHash-prim TSTSTR TSTSTR-LEN 0) (DEKHash TSTSTR))
		(expect-eqv "APHash" (APHash-prim TSTSTR TSTSTR-LEN 0) (APHash TSTSTR))
		(expect-eqv "CRCHash" (CRCHash-prim TSTSTR TSTSTR-LEN 0) (CRCHash TSTSTR))
		(expect-eqv "PHSFHash" (PHSFHash-prim TSTSTR TSTSTR-LEN 0) (PHSFHash TSTSTR))
		(expect-eqv "FNVHash" (FNVHash-prim TSTSTR TSTSTR-LEN 0) (FNVHash TSTSTR))
		(expect-eqv "FNVAHash" (FNVAHash-prim TSTSTR TSTSTR-LEN 0) (FNVAHash TSTSTR))
		(expect-eqv "BRPHash" (BRPHash-prim TSTSTR TSTSTR-LEN 0) (BRPHash TSTSTR))
		(expect-eqv "PYHash" (PYHash-prim TSTSTR TSTSTR-LEN 0) (PYHash TSTSTR))
		(expect-eqv "RJL3Hash" (RJL3Hash-prim TSTSTR TSTSTR-LEN 0) (RJL3Hash TSTSTR))
		(expect-eqv "ISPLHash" (ISPLHash-prim TSTSTR TSTSTR-LEN 0) (ISPLHash TSTSTR))
	)

	(test/suite "Hash Length Arg"

		(expect-eqv "RJMXHash" (RJMXHash-prim TSTSTR TSTSTR-HALF-LEN 0) (RJMXHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "TWMXHash" (TWMXHash-prim TSTSTR TSTSTR-HALF-LEN 0) (TWMXHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "TWSHMXHash" (TWSHMXHash-prim TSTSTR TSTSTR-HALF-LEN 0) (TWSHMXHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "TWSHMLMXHash" (TWSHMLMXHash-prim TSTSTR TSTSTR-HALF-LEN 0) (TWSHMLMXHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "TWMGMXHash" (TWMGMXHash-prim TSTSTR TSTSTR-HALF-LEN 0) (TWMGMXHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "RSHash" (RSHash-prim TSTSTR TSTSTR-HALF-LEN 0) (RSHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "JSHash" (JSHash-prim TSTSTR TSTSTR-HALF-LEN 0) (JSHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "PJWHash" (PJWHash-prim TSTSTR TSTSTR-HALF-LEN 0) (PJWHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "ELFHash" (ELFHash-prim TSTSTR TSTSTR-HALF-LEN 0) (ELFHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "BKDRHash" (BKDRHash-prim TSTSTR TSTSTR-HALF-LEN 0) (BKDRHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "SDBMHash" (SDBMHash-prim TSTSTR TSTSTR-HALF-LEN 0) (SDBMHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "DJBHash" (DJBHash-prim TSTSTR TSTSTR-HALF-LEN 0) (DJBHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "NDJBHash" (NDJBHash-prim TSTSTR TSTSTR-HALF-LEN 0) (NDJBHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "DEKHash" (DEKHash-prim TSTSTR TSTSTR-HALF-LEN 0) (DEKHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "APHash" (APHash-prim TSTSTR TSTSTR-HALF-LEN 0) (APHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "CRCHash" (CRCHash-prim TSTSTR TSTSTR-HALF-LEN 0) (CRCHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "PHSFHash" (PHSFHash-prim TSTSTR TSTSTR-HALF-LEN 0) (PHSFHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "FNVHash" (FNVHash-prim TSTSTR TSTSTR-HALF-LEN 0) (FNVHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "FNVAHash" (FNVAHash-prim TSTSTR TSTSTR-HALF-LEN 0) (FNVAHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "BRPHash" (BRPHash-prim TSTSTR TSTSTR-HALF-LEN 0) (BRPHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "PYHash" (PYHash-prim TSTSTR TSTSTR-HALF-LEN 0) (PYHash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "RJL3Hash" (RJL3Hash-prim TSTSTR TSTSTR-HALF-LEN 0) (RJL3Hash TSTSTR TSTSTR-HALF-LEN))
		(expect-eqv "ISPLHash" (ISPLHash-prim TSTSTR TSTSTR-HALF-LEN 0) (ISPLHash TSTSTR TSTSTR-HALF-LEN))
	)

	(test/suite "Digest"

		(expect-success "RJMXHash" (RJMXHash:digest TSTSTR))
		(expect-success "TWMXHash" (TWMXHash:digest TSTSTR))
		(expect-success "TWSHMXHash" (TWSHMXHash:digest TSTSTR))
		(expect-success "TWSHMLMXHash" (TWSHMLMXHash:digest TSTSTR))
		(expect-success "TWMGMXHash" (TWMGMXHash:digest TSTSTR))
		(expect-success "RSHash" (RSHash:digest TSTSTR))
		(expect-success "JSHash" (JSHash:digest TSTSTR))
		(expect-success "PJWHash" (PJWHash:digest TSTSTR))
		(expect-success "ELFHash" (ELFHash:digest TSTSTR))
		(expect-success "BKDRHash" (BKDRHash:digest TSTSTR))
		(expect-success "SDBMHash" (SDBMHash:digest TSTSTR))
		(expect-success "DJBHash" (DJBHash:digest TSTSTR))
		(expect-success "NDJBHash" (NDJBHash:digest TSTSTR))
		(expect-success "DEKHash" (DEKHash:digest TSTSTR))
		(expect-success "APHash" (APHash:digest TSTSTR))
		(expect-success "CRCHash" (CRCHash:digest TSTSTR))
		(expect-success "PHSFHash" (PHSFHash:digest TSTSTR))
		(expect-success "FNVHash" (FNVHash:digest TSTSTR))
		(expect-success "FNVAHash" (FNVAHash:digest TSTSTR))
		(expect-success "BRPHash" (BRPHash:digest TSTSTR))
		(expect-success "PYHash" (PYHash:digest TSTSTR))
		(expect-success "RJL3Hash" (RJL3Hash:digest TSTSTR))
		(expect-success "ISPLHash" (ISPLHash:digest TSTSTR))
	)
)

(define-test hashes-utils-test "Utilities"
  (initial
    (define tstr (make-string 4))
    (define bnd-hsh #f)
    (define str TSTSTR) )

  (expect-success "string" (unsigned-integer32-set! tstr (arithmetic-shift 1 31)))
  (test/eqv (unsigned-integer32-ref tstr) (arithmetic-shift 1 31))

  (expect-set! bnd-hsh (make-bounded-hash RJMXHash-prim))
  (expect-procedure bnd-hsh)
  (expect-number "number bound override" (bnd-hsh str 12345678901234567890))
  (expect-> "within bounds" 1456 (bnd-hsh str 1456))

  (expect-set! bnd-hsh (make-fixnum-bounded-hash RJMXHash-prim))
  (expect-procedure bnd-hsh)
  (expect-fixnum "fixnum bound override" (bnd-hsh str 1456))
  (expect-> "within bounds" 1456 (bnd-hsh str 1456))
)

;; This tests whether memory is being corrupted
;; There was a problem w/ syntax-case & RJL3Hash

(define-test hashes-random-test "RJL3Hash Idempotent?"
  (expect-eqv "1" 2110415480 (RJL3Hash TSTSTR))
  (expect-eqv "2" 2110415480 (RJL3Hash TSTSTR))
  (expect-eqv "3" 2110415480 (RJL3Hash TSTSTR))
  (expect-eqv "4" 2110415480 (RJL3Hash TSTSTR))
  (expect-eqv "5" 2110415480 (RJL3Hash TSTSTR))
  (expect-eqv "6" 2110415480 (RJL3Hash TSTSTR))
  (expect-eqv "7" 2110415480 (RJL3Hash TSTSTR))
  (expect-eqv "8" 2110415480 (RJL3Hash TSTSTR))
  (expect-eqv "9" 2110415480 (RJL3Hash TSTSTR))
)

(define-test hashes-utils-test "TWUserMixHash"
  (initial
    (define (mix key) key)
    (define usrmixhsh)
    (define hash-prim)
		(define hash)
		(define binary:digest)
		(define text:digest)
		(define prim:digest) )

    (expect-set! "TWUserMixHash Make" usrmixhsh (receive (make-TWUserMixHash mix #t)))
    (side-effect
      (set! hash-prim       (first usrmixhsh))
      (set! hash            (second usrmixhsh))
      (set! binary:digest   (third usrmixhsh))
      (set! text:digest     (fourth usrmixhsh))
      (set! prim:digest     (fifth usrmixhsh)) )

  (expect-eqv "TWUserMixHash-prim full-length" 543453076 (hash-prim TSTSTR TSTSTR-LEN 0))
  (expect-eqv "TWUserMixHash-prim half-length" 30057 (hash-prim TSTSTR TSTSTR-HALF-LEN 0))

  (expect-eqv "TWUserMixHash Hash" (hash-prim TSTSTR TSTSTR-LEN 0) (hash TSTSTR))
  (expect-eqv "TWUserMixHash Length Arg" (hash-prim TSTSTR TSTSTR-HALF-LEN 0) (hash TSTSTR TSTSTR-HALF-LEN))

  (expect-equal "TWUserMixHash Digest" "20646f94" (text:digest TSTSTR))
)

(define-test rabin-karp-test "Rabin-Karp Search"
	(initial
	  (define substrs '("quick" "foo" "brown" "dog" "skasfdskjsalksafnsalsfsdsdjkldsajlfsalsk"))
	  (define hashp)
	  (define rksp) )

  (expect-set! rksp (make-rabin-karp-string-search substrs))

  (expect-equal "W/O start & end" '("brown" (10 15)) (rksp TSTSTR))
  (expect-equal "W/ start & end" '("dog" (41 44)) (rksp TSTSTR 41 TSTSTR-LEN))

  (expect-set! hashp (make-fixnum-bounded-hash RJL3Hash-prim))
  (expect-set! rksp (make-rabin-karp-string-search substrs string=? hashp))

  (expect-equal "W/O start & end" '("brown" (10 15)) (rksp TSTSTR))
  (expect-equal "W/ start & end" '("dog" (41 44)) (rksp TSTSTR 41 TSTSTR-LEN))
)

;;;

(test::for-each (cut test::styler-set! <> test::output-style-human))
(run-test "Hashes Tests")

(test::forget!)
